@@ -1,30 +1,24 @@
+// SPDX-License-Identifier: GPL-3.0-or-later
+
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
-* Copyright 2013 - 2023, nymea GmbH
-* Contact: contact@nymea.io
+* Copyright (C) 2013 - 2024, nymea GmbH
+* Copyright (C) 2024 - 2025, chargebyte austria GmbH
 *
-* This file is part of nymea.
-* This project including source code and documentation is protected by
-* copyright law, and remains the property of nymea GmbH. All rights, including
-* reproduction, publication, editing and translation, are reserved. The use of
-* this project is subject to the terms of a license agreement to be concluded
-* with nymea GmbH in accordance with the terms of use of nymea GmbH, available
-* under https://nymea.io/license
+* This file is part of nymea-energy-plugin-nymea.
 *
-* GNU Lesser General Public License Usage
-* Alternatively, this project may be redistributed and/or modified under the
-* terms of the GNU Lesser General Public License as published by the Free
-* Software Foundation; version 3. This project is distributed in the hope that
-* it will be useful, but WITHOUT ANY WARRANTY; without even the implied
-* warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-* Lesser General Public License for more details.
+* nymea-energy-plugin-nymea.s free software: you can redistribute it and/or modify
+* it under the terms of the GNU General Public License as published by
+* the Free Software Foundation, either version 3 of the License, or
+* (at your option) any later version.
 *
-* You should have received a copy of the GNU Lesser General Public License
-* along with this project. If not, see <https://www.gnu.org/licenses/>.
+* nymea-energy-plugin-nymea.s distributed in the hope that it will be useful,
+* but WITHOUT ANY WARRANTY; without even the implied warranty of
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+* GNU General Public License for more details.
 *
-* For any further details and any questions please contact us under
-* contact@nymea.io or see our FAQ/Licensing Information on
-* https://nymea.io/license/faq
+* You should have received a copy of the GNU General Public License
+* along with nymea-energy-plugin-nymea. If not, see <https://www.gnu.org/licenses/>.
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
@@ -586,33 +580,35 @@ TimeFrames SpotMarketManager::fusePartialTimeFrames(const TimeFrames &timeFrames
 void SpotMarketManager::onProviderScoreEntriesChanged(const ScoreEntries &scoreEntries)
 {
     // Received new score entries, check if we habe to update the day based weighted scores.
-    QHash<QDate, ScoreEntries> weightedScores;
+    QHash<QDate, ScoreEntries> groupedScores;
     foreach (const ScoreEntry &scoreEntry, scoreEntries) {
-        if (!weightedScores.contains(scoreEntry.startDateTime().date())) {
-            // New day, new situation... let's fill in the data and weight them
-            weightedScores[scoreEntry.startDateTime().date()] = ScoreEntries({ scoreEntry });
-        } else {
-            weightedScores[scoreEntry.startDateTime().date()].append(scoreEntry);
-        }
+        groupedScores[scoreEntry.startDateTime().date()].append(scoreEntry);
     }
 
-    // Insert all received scored weighted into our peristant hash
-    foreach (const QDate &date, weightedScores.keys()) {
-        if (!m_weightedScores.contains(date)) {
-            m_weightedScores[date] = weightScoreEntries(weightedScores.value(date));
-        }
+    // Update all received score entries in the persistent hash (cache may contain partial data).
+    foreach (const QDate &date, groupedScores.keys()) {
+        const ScoreEntries weightedEntries = weightScoreEntries(groupedScores.value(date));
+        m_weightedScores[date] = weightedEntries;
+        qCDebug(dcNymeaEnergy()) << "SpotMarketManager: Weighted day"
+                                 << date.toString("dd.MM.yyyy")
+                                 << "with" << groupedScores.value(date).count()
+                                 << "entries (provider total" << scoreEntries.count()
+                                 << "weighted total" << weightedEntries.count() << ")";
     }
 
     // Clean up old
     QList<QDate> datesToRemove;
-    foreach (const QDate &date, m_weightedScores.keys()) {
-        if (date < scoreEntries.first().startDateTime().date()) {
-            datesToRemove.append(date);
-            continue;
-        }
+    if (!scoreEntries.isEmpty()) {
+        const QDate earliestDate = scoreEntries.first().startDateTime().date();
+        foreach (const QDate &date, m_weightedScores.keys()) {
+            if (date < earliestDate) {
+                datesToRemove.append(date);
+                continue;
+            }
 
-       // Weight the current data
-       //m_weightedScores[date] = weightScoreEntries(m_weightedScores.value(date));
+            // Weight the current data
+            //m_weightedScores[date] = weightScoreEntries(m_weightedScores.value(date));
+        }
     }
 
     // Remove the past... live goes on
@@ -623,4 +619,3 @@ void SpotMarketManager::onProviderScoreEntriesChanged(const ScoreEntries &scoreE
     qCDebug(dcNymeaEnergy()) << "SpotMarketManager: Score entries updated";
     emit scoreEntriesUpdated();
 }
-
